@@ -16,9 +16,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
-#ifndef NAUTILUS_VIEW_H
-#define NAUTILUS_VIEW_H
+#pragma once
 
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -26,6 +24,15 @@
 #include "nautilus-query.h"
 #include "nautilus-toolbar-menu-sections.h"
 
+enum
+{
+    NAUTILUS_VIEW_GRID_ID,
+    NAUTILUS_VIEW_LIST_ID,
+    NAUTILUS_VIEW_EMPTY_ID,
+    NAUTILUS_VIEW_OTHER_LOCATIONS_ID,
+    NAUTILUS_VIEW_INVALID_ID,
+};
+
 G_BEGIN_DECLS
 
 #define NAUTILUS_TYPE_VIEW (nautilus_view_get_type ())
@@ -36,15 +43,27 @@ struct _NautilusViewInterface
 {
         GTypeInterface parent;
 
-        /* The icon that represents the view */
-        GIcon*                          (*get_icon)                  (NautilusView         *view);
-
+        guint                           (*get_view_id)               (NautilusView         *view);
         /*
          * Returns the menu sections that should be shown in the toolbar menu
          * when this view is active. Implementations must not return %NULL
          */
         NautilusToolbarMenuSections *   (*get_toolbar_menu_sections) (NautilusView         *view);
 
+        /*
+         * Returns the menu for the background click of extensions.
+         */
+        GMenu *   (*get_extensions_background_menu) (NautilusView         *view);
+
+        void     (*set_extensions_background_menu) (NautilusView *view,
+                                                    GMenu        *menu);
+        /*
+         * Returns the menu for templates.
+         */
+        GMenu *   (*get_templates_menu) (NautilusView         *view);
+
+        void     (*set_templates_menu) (NautilusView *view,
+                                        GMenu        *menu);
         /* Current location of the view */
         GFile*                          (*get_location)              (NautilusView         *view);
         void                            (*set_location)              (NautilusView         *view,
@@ -67,7 +86,9 @@ struct _NautilusViewInterface
         gboolean                        (*is_searching)              (NautilusView         *view);
 };
 
-GIcon *                        nautilus_view_get_icon                  (NautilusView         *view);
+GIcon *                        nautilus_view_get_icon                  (guint                 view_id);
+
+guint                          nautilus_view_get_view_id               (NautilusView         *view);
 
 NautilusToolbarMenuSections *  nautilus_view_get_toolbar_menu_sections (NautilusView         *view);
 
@@ -90,6 +111,11 @@ gboolean                       nautilus_view_is_loading                (Nautilus
 
 gboolean                       nautilus_view_is_searching              (NautilusView         *view);
 
-G_END_DECLS
+void                           nautilus_view_set_templates_menu        (NautilusView *view,
+                                                                        GMenu        *menu);
+GMenu*                         nautilus_view_get_templates_menu        (NautilusView *view);
+void                           nautilus_view_set_extensions_background_menu (NautilusView *view,
+                                                                             GMenu        *menu);
+GMenu*                         nautilus_view_get_extensions_background_menu (NautilusView *view);
 
-#endif /* NAUTILUS_VIEW_H */
+G_END_DECLS
