@@ -793,15 +793,6 @@ update_cursor (NautilusWindow *window)
     }
 }
 
-void
-nautilus_window_hide_view_menu (NautilusWindow *window)
-{
-    GAction *menu_action;
-
-    menu_action = g_action_map_lookup_action (G_ACTION_MAP (window), "view-menu");
-    g_action_change_state (menu_action, g_variant_new_boolean (FALSE));
-}
-
 void
 nautilus_window_reset_menus (NautilusWindow *window)
 {
@@ -918,6 +909,7 @@ open_location_cb (NautilusWindow     *window,
                   GtkPlacesOpenFlags  open_flags)
 {
     NautilusWindowOpenFlags flags;
+    NautilusApplication *application;
 
     switch (open_flags)
     {
@@ -942,11 +934,12 @@ open_location_cb (NautilusWindow     *window,
         break;
     }
 
+    application = NAUTILUS_APPLICATION (g_application_get_default ());
     /* FIXME: We shouldn't need to provide the window, but seems gtk_application_get_active_window
      * is not working properly in GtkApplication, so we cannot rely on that...
      */
-    nautilus_application_open_location_full (NAUTILUS_APPLICATION (g_application_get_default ()),
-                                             location, flags, NULL, window, NULL);
+    nautilus_application_open_location_full (application, location, flags,
+                                             NULL, window, NULL);
 }
 
 static void
@@ -1161,7 +1154,8 @@ action_properties (GSimpleAction *action,
     file = nautilus_file_get (window->selected_file);
 
     list = g_list_append (NULL, file);
-    nautilus_properties_window_present (list, GTK_WIDGET (window), NULL);
+    nautilus_properties_window_present (list, GTK_WIDGET (window), NULL, NULL,
+                                        NULL);
     nautilus_file_list_free (list);
 
     g_clear_object (&window->selected_file);
